require 'rake/file_utils_ext'
require_relative 'core-ext'


#def bash_cmd(cmd)
#  # sh <<-EOS.strip_heredoc, {verbose: false}
#  sh <<-EOS << cmd
#/bin/bash -xeuo pipefail
#EOS
#end

# noinspection ALL
module Trop
  class Sh

    DEFAULT = false
    @@verbose = false
    @@debug = false

    def self.verbose?
      if Rake::FileUtilsExt.verbose_flag == true || (!ENV['VERBOSE'].nil? && ENV['VERBOSE'] == 'true') || !(ENV['DEBUG'].nil?)
        @@verbose = true
      end
      return !!@@verbose
    end

    def self.verbose!
      @@verbose = true
      ENV['VERBOSE'] = "true"
      return !!@@verbose
    end

    def self.silence_streams(*streams)
      @@on_hold = streams.collect {|stream| stream.dup}
      streams.each do |stream|
        stream.reopen(RUBY_PLATFORM =~ /mswin/ ? 'NUL:' : '/dev/null')
        stream.sync = true
      end
      yield
    ensure
      streams.each_with_index do |stream, i|
        stream.reopen(@@on_hold[i])
      end
    end


    def self.silent!
      @@verbose = false
      @@debug = false
      ENV['VERBOSE'] = nil
      ENV['DEBUG'] = nil
      return !!@@verbose && !!@@debug
    end

    def self.silent?
      return !(::Trop::Sh.verbose? || ::Trop::Sh.debug?)
    end

    def self.debug?
      @@debug = true unless ENV['DEBUG'].nil?
      @@debug = true if $DEBUG
      return @@debug
    end

    def self.debug!(mode = true)
      if mode.false?
        @@debug = false
        ENV['DEBUG'] = nil
      else
        @@debug = true
      end
      return @@debug
    end


    def self.systemlocal(cmds)
      cmd = ''
      return [-1, ''] if cmds.nil? || cmds.blank?

      if cmds.is_a? Array
        cmds.each {|s| cmd = cmd + s.to_s + ' ' unless s.blank?}
      else
        cmd = cmds.to_s
      end
      puts "commands: #{cmd} " if Trop::Sh::debug?
      return [-1, ''] if cmd.blank?
      puts " #{cmd} ".colorize(:green) if Trop::Sh::verbose?

      # execute system call with return in stdout
      stdout = `#{cmd}`
      # silent if successful or not verbose
      if $CHILD_STATUS.success?
        puts stdout.to_s.colorize(:green) if Trop::Sh::verbose?
        return [0, stdout]
      end
      # error case
      puts " #{cmd} ".colorize(:red)
      puts " #{stdout}".colorize(:red)
      [-1, stdout]
    end

    def self.cwd(*cmds)
      wd = Pathname.new(RAKE_RUN_DIR).join('output')
      Dir.chdir(wd) do
        ret, out = systemlocal(cmds)
        return [ret, out]
      end
    end

    def self.sh(*cmds)
      ret, out = systemlocal(cmds)
      return [ret, out]
    end
  end
end

