require_relative '../lib/trop'


namespace :gem do

  DEVNULL = Gem.win_platform? ? "NUL" : "/dev/null"

  task :gembox do
    Trop::Sh::silent!
    Dir.chdir('./pkg') do
      Trop::Sh::systemlocal 'gem inabox *.gem --host=http://localhost:9292 -o '
    end
  end

  task :next do
    # self.dirty_tree?
    # Are we in a dirty, dirty tree?
    Trop::Sh::silent!
    status,msg = Trop::Sh::systemlocal("git diff --no-ext-diff HEAD 2> #{DEVNULL}" )
    stat2,msg2 = Trop::Sh::systemlocal("git diff-index --cached HEAD 2> #{DEVNULL}")
    clean = (!status && !stat2) && msg.empty? && msg2.empty?
    clean = msg.empty? && msg2.empty?

    if !clean
      Trop::Sh::verbose!
      puts msg
      puts " repo is not clean please stash or commit"
    else
      vers1 = vers2 = nil
      Trop::Sh::silence_streams(STDOUT) {
        vers1 = Trop::GemVersion.increment_version
        vers2 = Trop::GemVersion.rewrite_gemspec
        Trop::GemVersion.commit_and_push {|g| g.add(Trop::VERSION_FILE) && g.add(Trop::GEMSPEC_NAME)}
      }

      puts "incr. version build and push #{vers1} #{vers2}"
      Trop::Sh::systemlocal('gem build trop.gemspec')
      #Trop::Sh::systemlocal('rake gembox')
    end
  end

  desc 'prepare deployment in public'
  task :deploy do
    `mkdir -p ./public`
    FileUtils.cp(%w[LICENSE.txt README.md README.rakes.md TODO.md], 'public', verbose: true)
  end

  desc "Clean *.gem* files from project root"
  task :clean do
    FileUtils.rm_f Dir.glob('*.gem*')
  end

  desc "sync version files"
  task :sync do
    Trop::GemVersion::update_version(Trop::GemVersion::next_version)
  end

  desc "Return and set next gem version"
  task :version do
    if (ENV.include?("set"))
      version = ENV['set'].gsub(/'/, '')
      unless Trop::GemVersion.version_format_valid?(version)
        raise "Version '#{version}' format invalid.  Must only contain digits separated by decimal points."
      end
      Trop::GemVersion.set_version(version)
      puts "Next version set at '#{version}'"
    else
      puts Trop::GemVersion.next_version
    end
  end
end
